export default class MatrixStack {
    constructor() {
        this.stack = [];

        this.restore();
    }

    getCurrentMatrix() {
        return glMatrix.mat4.clone(this.stack[this.stack.length - 1]);
    }

    setCurrentMatrix(matrix) {
        this.stack[this.stack.length - 1] = matrix;
    }

    restore() {
        this.stack.pop();

        if (this.stack.length == 0) {
            // stack is empty. place identity
            this.stack.push(glMatrix.mat4.create());
        }
    }

    clear() {
        this.stack = [];
        this.restore();
    }

    save() {
        this.stack.push(this.getCurrentMatrix());
    }

    translate(x, y, z) {
        let m = this.getCurrentMatrix();
        glMatrix.mat4.translate(m, m, [x, y, z]);
        this.setCurrentMatrix(m);
    }

    rotate(angle, axis) {
        let m = this.getCurrentMatrix();
        glMatrix.mat4.rotate(m, m, angle, axis);
        this.setCurrentMatrix(m);
    }

    scale(sx, sy, sz) {
        let m = this.getCurrentMatrix();
        glMatrix.mat4.scale(m, m, [sx, sy, sz]);
        this.setCurrentMatrix(m);
    }

    multiply(matrix) {
        let m = this.getCurrentMatrix();
        glMatrix.mat4.multiply(m, m, matrix);
        this.setCurrentMatrix(m);
    }

    multiplyLeft(matrix) {
        let m = this.getCurrentMatrix();
        glMatrix.mat4.multiply(m, matrix, m);
        this.setCurrentMatrix(m);
    }
}